/*******************************************************************************
* File Name: SSD1331_master.c
* Version 2.50
*
* Description:
*  This file provides all API functionality of the SPI Master component.
*
* Note:
*  None.
*
********************************************************************************
* Copyright 2008-2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "SSD1331_master_PVT.h"

#if(SSD1331_master_TX_SOFTWARE_BUF_ENABLED)
    volatile uint8 SSD1331_master_txBuffer[SSD1331_master_TX_BUFFER_SIZE];
    volatile uint8 SSD1331_master_txBufferFull;
    volatile uint8 SSD1331_master_txBufferRead;
    volatile uint8 SSD1331_master_txBufferWrite;
#endif /* (SSD1331_master_TX_SOFTWARE_BUF_ENABLED) */

#if(SSD1331_master_RX_SOFTWARE_BUF_ENABLED)
    volatile uint8 SSD1331_master_rxBuffer[SSD1331_master_RX_BUFFER_SIZE];
    volatile uint8 SSD1331_master_rxBufferFull;
    volatile uint8 SSD1331_master_rxBufferRead;
    volatile uint8 SSD1331_master_rxBufferWrite;
#endif /* (SSD1331_master_RX_SOFTWARE_BUF_ENABLED) */

uint8 SSD1331_master_initVar = 0u;

volatile uint8 SSD1331_master_swStatusTx;
volatile uint8 SSD1331_master_swStatusRx;


/*******************************************************************************
* Function Name: SSD1331_master_Init
********************************************************************************
*
* Summary:
*  Inits/Restores default SPIM configuration provided with customizer.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Side Effects:
*  When this function is called it initializes all of the necessary parameters
*  for execution. i.e. setting the initial interrupt mask, configuring the
*  interrupt service routine, configuring the bit-counter parameters and
*  clearing the FIFO and Status Register.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void SSD1331_master_Init(void) 
{
    /* Initialize the Bit counter */
    SSD1331_master_COUNTER_PERIOD_REG = SSD1331_master_BITCTR_INIT;

    /* Init TX ISR  */
    #if(0u != SSD1331_master_INTERNAL_TX_INT_ENABLED)
        CyIntDisable         (SSD1331_master_TX_ISR_NUMBER);
        CyIntSetPriority     (SSD1331_master_TX_ISR_NUMBER,  SSD1331_master_TX_ISR_PRIORITY);
        (void) CyIntSetVector(SSD1331_master_TX_ISR_NUMBER, &SSD1331_master_TX_ISR);
    #endif /* (0u != SSD1331_master_INTERNAL_TX_INT_ENABLED) */

    /* Init RX ISR  */
    #if(0u != SSD1331_master_INTERNAL_RX_INT_ENABLED)
        CyIntDisable         (SSD1331_master_RX_ISR_NUMBER);
        CyIntSetPriority     (SSD1331_master_RX_ISR_NUMBER,  SSD1331_master_RX_ISR_PRIORITY);
        (void) CyIntSetVector(SSD1331_master_RX_ISR_NUMBER, &SSD1331_master_RX_ISR);
    #endif /* (0u != SSD1331_master_INTERNAL_RX_INT_ENABLED) */

    /* Clear any stray data from the RX and TX FIFO */
    SSD1331_master_ClearFIFO();

    #if(SSD1331_master_RX_SOFTWARE_BUF_ENABLED)
        SSD1331_master_rxBufferFull  = 0u;
        SSD1331_master_rxBufferRead  = 0u;
        SSD1331_master_rxBufferWrite = 0u;
    #endif /* (SSD1331_master_RX_SOFTWARE_BUF_ENABLED) */

    #if(SSD1331_master_TX_SOFTWARE_BUF_ENABLED)
        SSD1331_master_txBufferFull  = 0u;
        SSD1331_master_txBufferRead  = 0u;
        SSD1331_master_txBufferWrite = 0u;
    #endif /* (SSD1331_master_TX_SOFTWARE_BUF_ENABLED) */

    (void) SSD1331_master_ReadTxStatus(); /* Clear Tx status and swStatusTx */
    (void) SSD1331_master_ReadRxStatus(); /* Clear Rx status and swStatusRx */

    /* Configure TX and RX interrupt mask */
    SSD1331_master_TX_STATUS_MASK_REG = SSD1331_master_TX_INIT_INTERRUPTS_MASK;
    SSD1331_master_RX_STATUS_MASK_REG = SSD1331_master_RX_INIT_INTERRUPTS_MASK;
}


/*******************************************************************************
* Function Name: SSD1331_master_Enable
********************************************************************************
*
* Summary:
*  Enable SPIM component.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
*******************************************************************************/
void SSD1331_master_Enable(void) 
{
    uint8 enableInterrupts;

    enableInterrupts = CyEnterCriticalSection();
    SSD1331_master_COUNTER_CONTROL_REG |= SSD1331_master_CNTR_ENABLE;
    SSD1331_master_TX_STATUS_ACTL_REG  |= SSD1331_master_INT_ENABLE;
    SSD1331_master_RX_STATUS_ACTL_REG  |= SSD1331_master_INT_ENABLE;
    CyExitCriticalSection(enableInterrupts);

    #if(0u != SSD1331_master_INTERNAL_CLOCK)
        SSD1331_master_IntClock_Enable();
    #endif /* (0u != SSD1331_master_INTERNAL_CLOCK) */

    SSD1331_master_EnableTxInt();
    SSD1331_master_EnableRxInt();
}


/*******************************************************************************
* Function Name: SSD1331_master_Start
********************************************************************************
*
* Summary:
*  Initialize and Enable the SPI Master component.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global variables:
*  SSD1331_master_initVar - used to check initial configuration, modified on
*  first function call.
*
* Theory:
*  Enable the clock input to enable operation.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void SSD1331_master_Start(void) 
{
    if(0u == SSD1331_master_initVar)
    {
        SSD1331_master_Init();
        SSD1331_master_initVar = 1u;
    }

    SSD1331_master_Enable();
}


/*******************************************************************************
* Function Name: SSD1331_master_Stop
********************************************************************************
*
* Summary:
*  Disable the SPI Master component.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Theory:
*  Disable the clock input to enable operation.
*
*******************************************************************************/
void SSD1331_master_Stop(void) 
{
    uint8 enableInterrupts;

    enableInterrupts = CyEnterCriticalSection();
    SSD1331_master_TX_STATUS_ACTL_REG &= ((uint8) ~SSD1331_master_INT_ENABLE);
    SSD1331_master_RX_STATUS_ACTL_REG &= ((uint8) ~SSD1331_master_INT_ENABLE);
    CyExitCriticalSection(enableInterrupts);

    #if(0u != SSD1331_master_INTERNAL_CLOCK)
        SSD1331_master_IntClock_Disable();
    #endif /* (0u != SSD1331_master_INTERNAL_CLOCK) */

    SSD1331_master_DisableTxInt();
    SSD1331_master_DisableRxInt();
}


/*******************************************************************************
* Function Name: SSD1331_master_EnableTxInt
********************************************************************************
*
* Summary:
*  Enable internal Tx interrupt generation.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Theory:
*  Enable the internal Tx interrupt output -or- the interrupt component itself.
*
*******************************************************************************/
void SSD1331_master_EnableTxInt(void) 
{
    #if(0u != SSD1331_master_INTERNAL_TX_INT_ENABLED)
        CyIntEnable(SSD1331_master_TX_ISR_NUMBER);
    #endif /* (0u != SSD1331_master_INTERNAL_TX_INT_ENABLED) */
}


/*******************************************************************************
* Function Name: SSD1331_master_EnableRxInt
********************************************************************************
*
* Summary:
*  Enable internal Rx interrupt generation.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Theory:
*  Enable the internal Rx interrupt output -or- the interrupt component itself.
*
*******************************************************************************/
void SSD1331_master_EnableRxInt(void) 
{
    #if(0u != SSD1331_master_INTERNAL_RX_INT_ENABLED)
        CyIntEnable(SSD1331_master_RX_ISR_NUMBER);
    #endif /* (0u != SSD1331_master_INTERNAL_RX_INT_ENABLED) */
}


/*******************************************************************************
* Function Name: SSD1331_master_DisableTxInt
********************************************************************************
*
* Summary:
*  Disable internal Tx interrupt generation.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Theory:
*  Disable the internal Tx interrupt output -or- the interrupt component itself.
*
*******************************************************************************/
void SSD1331_master_DisableTxInt(void) 
{
    #if(0u != SSD1331_master_INTERNAL_TX_INT_ENABLED)
        CyIntDisable(SSD1331_master_TX_ISR_NUMBER);
    #endif /* (0u != SSD1331_master_INTERNAL_TX_INT_ENABLED) */
}


/*******************************************************************************
* Function Name: SSD1331_master_DisableRxInt
********************************************************************************
*
* Summary:
*  Disable internal Rx interrupt generation.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Theory:
*  Disable the internal Rx interrupt output -or- the interrupt component itself.
*
*******************************************************************************/
void SSD1331_master_DisableRxInt(void) 
{
    #if(0u != SSD1331_master_INTERNAL_RX_INT_ENABLED)
        CyIntDisable(SSD1331_master_RX_ISR_NUMBER);
    #endif /* (0u != SSD1331_master_INTERNAL_RX_INT_ENABLED) */
}


/*******************************************************************************
* Function Name: SSD1331_master_SetTxInterruptMode
********************************************************************************
*
* Summary:
*  Configure which status bits trigger an interrupt event.
*
* Parameters:
*  intSrc: An or'd combination of the desired status bit masks (defined in the
*  header file).
*
* Return:
*  None.
*
* Theory:
*  Enables the output of specific status bits to the interrupt controller.
*
*******************************************************************************/
void SSD1331_master_SetTxInterruptMode(uint8 intSrc) 
{
    SSD1331_master_TX_STATUS_MASK_REG = intSrc;
}


/*******************************************************************************
* Function Name: SSD1331_master_SetRxInterruptMode
********************************************************************************
*
* Summary:
*  Configure which status bits trigger an interrupt event.
*
* Parameters:
*  intSrc: An or'd combination of the desired status bit masks (defined in the
*  header file).
*
* Return:
*  None.
*
* Theory:
*  Enables the output of specific status bits to the interrupt controller.
*
*******************************************************************************/
void SSD1331_master_SetRxInterruptMode(uint8 intSrc) 
{
    SSD1331_master_RX_STATUS_MASK_REG  = intSrc;
}


/*******************************************************************************
* Function Name: SSD1331_master_ReadTxStatus
********************************************************************************
*
* Summary:
*  Read the Tx status register for the component.
*
* Parameters:
*  None.
*
* Return:
*  Contents of the Tx status register.
*
* Global variables:
*  SSD1331_master_swStatusTx - used to store in software status register,
*  modified every function call - resets to zero.
*
* Theory:
*  Allows the user and the API to read the Tx status register for error
*  detection and flow control.
*
* Side Effects:
*  Clear Tx status register of the component.
*
* Reentrant:
*  No.
*
*******************************************************************************/
uint8 SSD1331_master_ReadTxStatus(void) 
{
    uint8 tmpStatus;

    #if(SSD1331_master_TX_SOFTWARE_BUF_ENABLED)
        /* Disable TX interrupt to protect global veriables */
        SSD1331_master_DisableTxInt();

        tmpStatus = SSD1331_master_GET_STATUS_TX(SSD1331_master_swStatusTx);
        SSD1331_master_swStatusTx = 0u;

        SSD1331_master_EnableTxInt();

    #else

        tmpStatus = SSD1331_master_TX_STATUS_REG;

    #endif /* (SSD1331_master_TX_SOFTWARE_BUF_ENABLED) */

    return(tmpStatus);
}


/*******************************************************************************
* Function Name: SSD1331_master_ReadRxStatus
********************************************************************************
*
* Summary:
*  Read the Rx status register for the component.
*
* Parameters:
*  None.
*
* Return:
*  Contents of the Rx status register.
*
* Global variables:
*  SSD1331_master_swStatusRx - used to store in software Rx status register,
*  modified every function call - resets to zero.
*
* Theory:
*  Allows the user and the API to read the Rx status register for error
*  detection and flow control.
*
* Side Effects:
*  Clear Rx status register of the component.
*
* Reentrant:
*  No.
*
*******************************************************************************/
uint8 SSD1331_master_ReadRxStatus(void) 
{
    uint8 tmpStatus;

    #if(SSD1331_master_RX_SOFTWARE_BUF_ENABLED)
        /* Disable RX interrupt to protect global veriables */
        SSD1331_master_DisableRxInt();

        tmpStatus = SSD1331_master_GET_STATUS_RX(SSD1331_master_swStatusRx);
        SSD1331_master_swStatusRx = 0u;

        SSD1331_master_EnableRxInt();

    #else

        tmpStatus = SSD1331_master_RX_STATUS_REG;

    #endif /* (SSD1331_master_RX_SOFTWARE_BUF_ENABLED) */

    return(tmpStatus);
}


/*******************************************************************************
* Function Name: SSD1331_master_WriteTxData
********************************************************************************
*
* Summary:
*  Write a byte of data to be sent across the SPI.
*
* Parameters:
*  txDataByte: The data value to send across the SPI.
*
* Return:
*  None.
*
* Global variables:
*  SSD1331_master_txBufferWrite - used for the account of the bytes which
*  have been written down in the TX software buffer, modified every function
*  call if TX Software Buffer is used.
*  SSD1331_master_txBufferRead - used for the account of the bytes which
*  have been read from the TX software buffer.
*  SSD1331_master_txBuffer[SSD1331_master_TX_BUFFER_SIZE] - used to store
*  data to sending, modified every function call if TX Software Buffer is used.
*
* Theory:
*  Allows the user to transmit any byte of data in a single transfer.
*
* Side Effects:
*  If this function is called again before the previous byte is finished then
*  the next byte will be appended to the transfer with no time between
*  the byte transfers. Clear Tx status register of the component.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void SSD1331_master_WriteTxData(uint8 txData) 
{
    #if(SSD1331_master_TX_SOFTWARE_BUF_ENABLED)

        uint8 tempStatus;
        uint8 tmpTxBufferRead;

        /* Block if TX buffer is FULL: don't overwrite */
        do
        {
            tmpTxBufferRead = SSD1331_master_txBufferRead;
            if(0u == tmpTxBufferRead)
            {
                tmpTxBufferRead = (SSD1331_master_TX_BUFFER_SIZE - 1u);
            }
            else
            {
                tmpTxBufferRead--;
            }

        }while(tmpTxBufferRead == SSD1331_master_txBufferWrite);

        /* Disable TX interrupt to protect global veriables */
        SSD1331_master_DisableTxInt();

        tempStatus = SSD1331_master_GET_STATUS_TX(SSD1331_master_swStatusTx);
        SSD1331_master_swStatusTx = tempStatus;


        if((SSD1331_master_txBufferRead == SSD1331_master_txBufferWrite) &&
           (0u != (SSD1331_master_swStatusTx & SSD1331_master_STS_TX_FIFO_NOT_FULL)))
        {
            /* Put data element into the TX FIFO */
            CY_SET_REG8(SSD1331_master_TXDATA_PTR, txData);
        }
        else
        {
            /* Add to the TX software buffer */
            SSD1331_master_txBufferWrite++;
            if(SSD1331_master_txBufferWrite >= SSD1331_master_TX_BUFFER_SIZE)
            {
                SSD1331_master_txBufferWrite = 0u;
            }

            if(SSD1331_master_txBufferWrite == SSD1331_master_txBufferRead)
            {
                SSD1331_master_txBufferRead++;
                if(SSD1331_master_txBufferRead >= SSD1331_master_TX_BUFFER_SIZE)
                {
                    SSD1331_master_txBufferRead = 0u;
                }
                SSD1331_master_txBufferFull = 1u;
            }

            SSD1331_master_txBuffer[SSD1331_master_txBufferWrite] = txData;

            SSD1331_master_TX_STATUS_MASK_REG |= SSD1331_master_STS_TX_FIFO_NOT_FULL;
        }

        SSD1331_master_EnableTxInt();

    #else
        /* Wait until TX FIFO has a place */
        while(0u == (SSD1331_master_TX_STATUS_REG & SSD1331_master_STS_TX_FIFO_NOT_FULL))
        {
        }

        /* Put data element into the TX FIFO */
        CY_SET_REG8(SSD1331_master_TXDATA_PTR, txData);

    #endif /* (SSD1331_master_TX_SOFTWARE_BUF_ENABLED) */
}


/*******************************************************************************
* Function Name: SSD1331_master_ReadRxData
********************************************************************************
*
* Summary:
*  Read the next byte of data received across the SPI.
*
* Parameters:
*  None.
*
* Return:
*  The next byte of data read from the FIFO.
*
* Global variables:
*  SSD1331_master_rxBufferWrite - used for the account of the bytes which
*  have been written down in the RX software buffer.
*  SSD1331_master_rxBufferRead - used for the account of the bytes which
*  have been read from the RX software buffer, modified every function
*  call if RX Software Buffer is used.
*  SSD1331_master_rxBuffer[SSD1331_master_RX_BUFFER_SIZE] - used to store
*  received data.
*
* Theory:
*  Allows the user to read a byte of data received.
*
* Side Effects:
*  Will return invalid data if the FIFO is empty. The user should Call
*  GetRxBufferSize() and if it returns a non-zero value then it is safe to call
*  ReadByte() function.
*
* Reentrant:
*  No.
*
*******************************************************************************/
uint8 SSD1331_master_ReadRxData(void) 
{
    uint8 rxData;

    #if(SSD1331_master_RX_SOFTWARE_BUF_ENABLED)

        /* Disable RX interrupt to protect global veriables */
        SSD1331_master_DisableRxInt();

        if(SSD1331_master_rxBufferRead != SSD1331_master_rxBufferWrite)
        {
            if(0u == SSD1331_master_rxBufferFull)
            {
                SSD1331_master_rxBufferRead++;
                if(SSD1331_master_rxBufferRead >= SSD1331_master_RX_BUFFER_SIZE)
                {
                    SSD1331_master_rxBufferRead = 0u;
                }
            }
            else
            {
                SSD1331_master_rxBufferFull = 0u;
            }
        }

        rxData = SSD1331_master_rxBuffer[SSD1331_master_rxBufferRead];

        SSD1331_master_EnableRxInt();

    #else

        rxData = CY_GET_REG8(SSD1331_master_RXDATA_PTR);

    #endif /* (SSD1331_master_RX_SOFTWARE_BUF_ENABLED) */

    return(rxData);
}


/*******************************************************************************
* Function Name: SSD1331_master_GetRxBufferSize
********************************************************************************
*
* Summary:
*  Returns the number of bytes/words of data currently held in the RX buffer.
*  If RX Software Buffer not used then function return 0 if FIFO empty or 1 if
*  FIFO not empty. In another case function return size of RX Software Buffer.
*
* Parameters:
*  None.
*
* Return:
*  Integer count of the number of bytes/words in the RX buffer.
*
* Global variables:
*  SSD1331_master_rxBufferWrite - used for the account of the bytes which
*  have been written down in the RX software buffer.
*  SSD1331_master_rxBufferRead - used for the account of the bytes which
*  have been read from the RX software buffer.
*
* Side Effects:
*  Clear status register of the component.
*
*******************************************************************************/
uint8 SSD1331_master_GetRxBufferSize(void) 
{
    uint8 size;

    #if(SSD1331_master_RX_SOFTWARE_BUF_ENABLED)

        /* Disable RX interrupt to protect global veriables */
        SSD1331_master_DisableRxInt();

        if(SSD1331_master_rxBufferRead == SSD1331_master_rxBufferWrite)
        {
            size = 0u;
        }
        else if(SSD1331_master_rxBufferRead < SSD1331_master_rxBufferWrite)
        {
            size = (SSD1331_master_rxBufferWrite - SSD1331_master_rxBufferRead);
        }
        else
        {
            size = (SSD1331_master_RX_BUFFER_SIZE - SSD1331_master_rxBufferRead) + SSD1331_master_rxBufferWrite;
        }

        SSD1331_master_EnableRxInt();

    #else

        /* We can only know if there is data in the RX FIFO */
        size = (0u != (SSD1331_master_RX_STATUS_REG & SSD1331_master_STS_RX_FIFO_NOT_EMPTY)) ? 1u : 0u;

    #endif /* (SSD1331_master_TX_SOFTWARE_BUF_ENABLED) */

    return(size);
}


/*******************************************************************************
* Function Name: SSD1331_master_GetTxBufferSize
********************************************************************************
*
* Summary:
*  Returns the number of bytes/words of data currently held in the TX buffer.
*  If TX Software Buffer not used then function return 0 - if FIFO empty, 1 - if
*  FIFO not full, 4 - if FIFO full. In another case function return size of TX
*  Software Buffer.
*
* Parameters:
*  None.
*
* Return:
*  Integer count of the number of bytes/words in the TX buffer.
*
* Global variables:
*  SSD1331_master_txBufferWrite - used for the account of the bytes which
*  have been written down in the TX software buffer.
*  SSD1331_master_txBufferRead - used for the account of the bytes which
*  have been read from the TX software buffer.
*
* Side Effects:
*  Clear status register of the component.
*
*******************************************************************************/
uint8  SSD1331_master_GetTxBufferSize(void) 
{
    uint8 size;

    #if(SSD1331_master_TX_SOFTWARE_BUF_ENABLED)
        /* Disable TX interrupt to protect global veriables */
        SSD1331_master_DisableTxInt();

        if(SSD1331_master_txBufferRead == SSD1331_master_txBufferWrite)
        {
            size = 0u;
        }
        else if(SSD1331_master_txBufferRead < SSD1331_master_txBufferWrite)
        {
            size = (SSD1331_master_txBufferWrite - SSD1331_master_txBufferRead);
        }
        else
        {
            size = (SSD1331_master_TX_BUFFER_SIZE - SSD1331_master_txBufferRead) + SSD1331_master_txBufferWrite;
        }

        SSD1331_master_EnableTxInt();

    #else

        size = SSD1331_master_TX_STATUS_REG;

        if(0u != (size & SSD1331_master_STS_TX_FIFO_EMPTY))
        {
            size = 0u;
        }
        else if(0u != (size & SSD1331_master_STS_TX_FIFO_NOT_FULL))
        {
            size = 1u;
        }
        else
        {
            size = SSD1331_master_FIFO_SIZE;
        }

    #endif /* (SSD1331_master_TX_SOFTWARE_BUF_ENABLED) */

    return(size);
}


/*******************************************************************************
* Function Name: SSD1331_master_ClearRxBuffer
********************************************************************************
*
* Summary:
*  Clear the RX RAM buffer by setting the read and write pointers both to zero.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global variables:
*  SSD1331_master_rxBufferWrite - used for the account of the bytes which
*  have been written down in the RX software buffer, modified every function
*  call - resets to zero.
*  SSD1331_master_rxBufferRead - used for the account of the bytes which
*  have been read from the RX software buffer, modified every function call -
*  resets to zero.
*
* Theory:
*  Setting the pointers to zero makes the system believe there is no data to
*  read and writing will resume at address 0 overwriting any data that may have
*  remained in the RAM.
*
* Side Effects:
*  Any received data not read from the RAM buffer will be lost when overwritten.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void SSD1331_master_ClearRxBuffer(void) 
{
    /* Clear Hardware RX FIFO */
    while(0u !=(SSD1331_master_RX_STATUS_REG & SSD1331_master_STS_RX_FIFO_NOT_EMPTY))
    {
        (void) CY_GET_REG8(SSD1331_master_RXDATA_PTR);
    }

    #if(SSD1331_master_RX_SOFTWARE_BUF_ENABLED)
        /* Disable RX interrupt to protect global veriables */
        SSD1331_master_DisableRxInt();

        SSD1331_master_rxBufferFull  = 0u;
        SSD1331_master_rxBufferRead  = 0u;
        SSD1331_master_rxBufferWrite = 0u;

        SSD1331_master_EnableRxInt();
    #endif /* (SSD1331_master_RX_SOFTWARE_BUF_ENABLED) */
}


/*******************************************************************************
* Function Name: SSD1331_master_ClearTxBuffer
********************************************************************************
*
* Summary:
*  Clear the TX RAM buffer by setting the read and write pointers both to zero.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global variables:
*  SSD1331_master_txBufferWrite - used for the account of the bytes which
*  have been written down in the TX software buffer, modified every function
*  call - resets to zero.
*  SSD1331_master_txBufferRead - used for the account of the bytes which
*  have been read from the TX software buffer, modified every function call -
*  resets to zero.
*
* Theory:
*  Setting the pointers to zero makes the system believe there is no data to
*  read and writing will resume at address 0 overwriting any data that may have
*  remained in the RAM.
*
* Side Effects:
*  Any data not yet transmitted from the RAM buffer will be lost when
*  overwritten.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void SSD1331_master_ClearTxBuffer(void) 
{
    uint8 enableInterrupts;

    enableInterrupts = CyEnterCriticalSection();
    /* Clear TX FIFO */
    SSD1331_master_AUX_CONTROL_DP0_REG |= ((uint8)  SSD1331_master_TX_FIFO_CLR);
    SSD1331_master_AUX_CONTROL_DP0_REG &= ((uint8) ~SSD1331_master_TX_FIFO_CLR);

    #if(SSD1331_master_USE_SECOND_DATAPATH)
        /* Clear TX FIFO for 2nd Datapath */
        SSD1331_master_AUX_CONTROL_DP1_REG |= ((uint8)  SSD1331_master_TX_FIFO_CLR);
        SSD1331_master_AUX_CONTROL_DP1_REG &= ((uint8) ~SSD1331_master_TX_FIFO_CLR);
    #endif /* (SSD1331_master_USE_SECOND_DATAPATH) */
    CyExitCriticalSection(enableInterrupts);

    #if(SSD1331_master_TX_SOFTWARE_BUF_ENABLED)
        /* Disable TX interrupt to protect global veriables */
        SSD1331_master_DisableTxInt();

        SSD1331_master_txBufferFull  = 0u;
        SSD1331_master_txBufferRead  = 0u;
        SSD1331_master_txBufferWrite = 0u;

        /* Buffer is EMPTY: disable TX FIFO NOT FULL interrupt */
        SSD1331_master_TX_STATUS_MASK_REG &= ((uint8) ~SSD1331_master_STS_TX_FIFO_NOT_FULL);

        SSD1331_master_EnableTxInt();
    #endif /* (SSD1331_master_TX_SOFTWARE_BUF_ENABLED) */
}


#if(0u != SSD1331_master_BIDIRECTIONAL_MODE)
    /*******************************************************************************
    * Function Name: SSD1331_master_TxEnable
    ********************************************************************************
    *
    * Summary:
    *  If the SPI master is configured to use a single bi-directional pin then this
    *  will set the bi-directional pin to transmit.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    *******************************************************************************/
    void SSD1331_master_TxEnable(void) 
    {
        SSD1331_master_CONTROL_REG |= SSD1331_master_CTRL_TX_SIGNAL_EN;
    }


    /*******************************************************************************
    * Function Name: SSD1331_master_TxDisable
    ********************************************************************************
    *
    * Summary:
    *  If the SPI master is configured to use a single bi-directional pin then this
    *  will set the bi-directional pin to receive.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    *******************************************************************************/
    void SSD1331_master_TxDisable(void) 
    {
        SSD1331_master_CONTROL_REG &= ((uint8) ~SSD1331_master_CTRL_TX_SIGNAL_EN);
    }

#endif /* (0u != SSD1331_master_BIDIRECTIONAL_MODE) */


/*******************************************************************************
* Function Name: SSD1331_master_PutArray
********************************************************************************
*
* Summary:
*  Write available data from ROM/RAM to the TX buffer while space is available
*  in the TX buffer. Keep trying until all data is passed to the TX buffer.
*
* Parameters:
*  *buffer: Pointer to the location in RAM containing the data to send
*  byteCount: The number of bytes to move to the transmit buffer.
*
* Return:
*  None.
*
* Side Effects:
*  Will stay in this routine until all data has been sent.  May get locked in
*  this loop if data is not being initiated by the master if there is not
*  enough room in the TX FIFO.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void SSD1331_master_PutArray(const uint8 buffer[], uint8 byteCount)
                                                                          
{
    uint8 bufIndex;

    bufIndex = 0u;

    while(byteCount > 0u)
    {
        SSD1331_master_WriteTxData(buffer[bufIndex]);
        bufIndex++;
        byteCount--;
    }
}


/*******************************************************************************
* Function Name: SSD1331_master_ClearFIFO
********************************************************************************
*
* Summary:
*  Clear the RX and TX FIFO's of all data for a fresh start.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Side Effects:
*  Clear status register of the component.
*
*******************************************************************************/
void SSD1331_master_ClearFIFO(void) 
{
    uint8 enableInterrupts;

    /* Clear Hardware RX FIFO */
    while(0u !=(SSD1331_master_RX_STATUS_REG & SSD1331_master_STS_RX_FIFO_NOT_EMPTY))
    {
        (void) CY_GET_REG8(SSD1331_master_RXDATA_PTR);
    }

    enableInterrupts = CyEnterCriticalSection();
    /* Clear TX FIFO */
    SSD1331_master_AUX_CONTROL_DP0_REG |= ((uint8)  SSD1331_master_TX_FIFO_CLR);
    SSD1331_master_AUX_CONTROL_DP0_REG &= ((uint8) ~SSD1331_master_TX_FIFO_CLR);

    #if(SSD1331_master_USE_SECOND_DATAPATH)
        /* Clear TX FIFO for 2nd Datapath */
        SSD1331_master_AUX_CONTROL_DP1_REG |= ((uint8)  SSD1331_master_TX_FIFO_CLR);
        SSD1331_master_AUX_CONTROL_DP1_REG &= ((uint8) ~SSD1331_master_TX_FIFO_CLR);
    #endif /* (SSD1331_master_USE_SECOND_DATAPATH) */
    CyExitCriticalSection(enableInterrupts);
}


/* Following functions are for version Compatibility, they are obsolete.
*  Please do not use it in new projects.
*/


/*******************************************************************************
* Function Name: SSD1331_master_EnableInt
********************************************************************************
*
* Summary:
*  Enable internal interrupt generation.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Theory:
*  Enable the internal interrupt output -or- the interrupt component itself.
*
*******************************************************************************/
void SSD1331_master_EnableInt(void) 
{
    SSD1331_master_EnableRxInt();
    SSD1331_master_EnableTxInt();
}


/*******************************************************************************
* Function Name: SSD1331_master_DisableInt
********************************************************************************
*
* Summary:
*  Disable internal interrupt generation.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Theory:
*  Disable the internal interrupt output -or- the interrupt component itself.
*
*******************************************************************************/
void SSD1331_master_DisableInt(void) 
{
    SSD1331_master_DisableTxInt();
    SSD1331_master_DisableRxInt();
}


/*******************************************************************************
* Function Name: SSD1331_master_SetInterruptMode
********************************************************************************
*
* Summary:
*  Configure which status bits trigger an interrupt event.
*
* Parameters:
*  intSrc: An or'd combination of the desired status bit masks (defined in the
*  header file).
*
* Return:
*  None.
*
* Theory:
*  Enables the output of specific status bits to the interrupt controller.
*
*******************************************************************************/
void SSD1331_master_SetInterruptMode(uint8 intSrc) 
{
    SSD1331_master_TX_STATUS_MASK_REG  = (intSrc & ((uint8) ~SSD1331_master_STS_SPI_IDLE));
    SSD1331_master_RX_STATUS_MASK_REG  =  intSrc;
}


/*******************************************************************************
* Function Name: SSD1331_master_ReadStatus
********************************************************************************
*
* Summary:
*  Read the status register for the component.
*
* Parameters:
*  None.
*
* Return:
*  Contents of the status register.
*
* Global variables:
*  SSD1331_master_swStatus - used to store in software status register,
*  modified every function call - resets to zero.
*
* Theory:
*  Allows the user and the API to read the status register for error detection
*  and flow control.
*
* Side Effects:
*  Clear status register of the component.
*
* Reentrant:
*  No.
*
*******************************************************************************/
uint8 SSD1331_master_ReadStatus(void) 
{
    uint8 tmpStatus;

    #if(SSD1331_master_TX_SOFTWARE_BUF_ENABLED || SSD1331_master_RX_SOFTWARE_BUF_ENABLED)

        SSD1331_master_DisableInt();

        tmpStatus  = SSD1331_master_GET_STATUS_RX(SSD1331_master_swStatusRx);
        tmpStatus |= SSD1331_master_GET_STATUS_TX(SSD1331_master_swStatusTx);
        tmpStatus &= ((uint8) ~SSD1331_master_STS_SPI_IDLE);

        SSD1331_master_swStatusTx = 0u;
        SSD1331_master_swStatusRx = 0u;

        SSD1331_master_EnableInt();

    #else

        tmpStatus  = SSD1331_master_RX_STATUS_REG;
        tmpStatus |= SSD1331_master_TX_STATUS_REG;
        tmpStatus &= ((uint8) ~SSD1331_master_STS_SPI_IDLE);

    #endif /* (SSD1331_master_TX_SOFTWARE_BUF_ENABLED || SSD1331_master_RX_SOFTWARE_BUF_ENABLED) */

    return(tmpStatus);
}


/* [] END OF FILE */
