/*******************************************************************************
* File Name: SSD1331_master_INT.c
* Version 2.50
*
* Description:
*  This file provides all Interrupt Service Routine (ISR) for the SPI Master
*  component.
*
* Note:
*  None.
*
********************************************************************************
* Copyright 2008-2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "SSD1331_master_PVT.h"
#include "cyapicallbacks.h"

/* User code required at start of ISR */
/* `#START SSD1331_master_ISR_START_DEF` */

/* `#END` */


/*******************************************************************************
* Function Name: SSD1331_master_TX_ISR
********************************************************************************
*
* Summary:
*  Interrupt Service Routine for TX portion of the SPI Master.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global variables:
*  SSD1331_master_txBufferWrite - used for the account of the bytes which
*  have been written down in the TX software buffer.
*  SSD1331_master_txBufferRead - used for the account of the bytes which
*  have been read from the TX software buffer, modified when exist data to
*  sending and FIFO Not Full.
*  SSD1331_master_txBuffer[SSD1331_master_TX_BUFFER_SIZE] - used to store
*  data to sending.
*  All described above Global variables are used when Software Buffer is used.
*
*******************************************************************************/
CY_ISR(SSD1331_master_TX_ISR)
{
    #if(SSD1331_master_TX_SOFTWARE_BUF_ENABLED)
        uint8 tmpStatus;
    #endif /* (SSD1331_master_TX_SOFTWARE_BUF_ENABLED) */

    #ifdef SSD1331_master_TX_ISR_ENTRY_CALLBACK
        SSD1331_master_TX_ISR_EntryCallback();
    #endif /* SSD1331_master_TX_ISR_ENTRY_CALLBACK */

    /* User code required at start of ISR */
    /* `#START SSD1331_master_TX_ISR_START` */

    /* `#END` */
    
    #if(SSD1331_master_TX_SOFTWARE_BUF_ENABLED)
        /* Check if TX data buffer is not empty and there is space in TX FIFO */
        while(SSD1331_master_txBufferRead != SSD1331_master_txBufferWrite)
        {
            tmpStatus = SSD1331_master_GET_STATUS_TX(SSD1331_master_swStatusTx);
            SSD1331_master_swStatusTx = tmpStatus;

            if(0u != (SSD1331_master_swStatusTx & SSD1331_master_STS_TX_FIFO_NOT_FULL))
            {
                if(0u == SSD1331_master_txBufferFull)
                {
                   SSD1331_master_txBufferRead++;

                    if(SSD1331_master_txBufferRead >= SSD1331_master_TX_BUFFER_SIZE)
                    {
                        SSD1331_master_txBufferRead = 0u;
                    }
                }
                else
                {
                    SSD1331_master_txBufferFull = 0u;
                }

                /* Put data element into the TX FIFO */
                CY_SET_REG8(SSD1331_master_TXDATA_PTR, 
                                             SSD1331_master_txBuffer[SSD1331_master_txBufferRead]);
            }
            else
            {
                break;
            }
        }

        if(SSD1331_master_txBufferRead == SSD1331_master_txBufferWrite)
        {
            /* TX Buffer is EMPTY: disable interrupt on TX NOT FULL */
            SSD1331_master_TX_STATUS_MASK_REG &= ((uint8) ~SSD1331_master_STS_TX_FIFO_NOT_FULL);
        }

    #endif /* (SSD1331_master_TX_SOFTWARE_BUF_ENABLED) */

    /* User code required at end of ISR (Optional) */
    /* `#START SSD1331_master_TX_ISR_END` */

    /* `#END` */
    
    #ifdef SSD1331_master_TX_ISR_EXIT_CALLBACK
        SSD1331_master_TX_ISR_ExitCallback();
    #endif /* SSD1331_master_TX_ISR_EXIT_CALLBACK */
}


/*******************************************************************************
* Function Name: SSD1331_master_RX_ISR
********************************************************************************
*
* Summary:
*  Interrupt Service Routine for RX portion of the SPI Master.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global variables:
*  SSD1331_master_rxBufferWrite - used for the account of the bytes which
*  have been written down in the RX software buffer modified when FIFO contains
*  new data.
*  SSD1331_master_rxBufferRead - used for the account of the bytes which
*  have been read from the RX software buffer, modified when overflow occurred.
*  SSD1331_master_rxBuffer[SSD1331_master_RX_BUFFER_SIZE] - used to store
*  received data, modified when FIFO contains new data.
*  All described above Global variables are used when Software Buffer is used.
*
*******************************************************************************/
CY_ISR(SSD1331_master_RX_ISR)
{
    #if(SSD1331_master_RX_SOFTWARE_BUF_ENABLED)
        uint8 tmpStatus;
        uint8 rxData;
    #endif /* (SSD1331_master_RX_SOFTWARE_BUF_ENABLED) */

    #ifdef SSD1331_master_RX_ISR_ENTRY_CALLBACK
        SSD1331_master_RX_ISR_EntryCallback();
    #endif /* SSD1331_master_RX_ISR_ENTRY_CALLBACK */

    /* User code required at start of ISR */
    /* `#START SSD1331_master_RX_ISR_START` */

    /* `#END` */
    
    #if(SSD1331_master_RX_SOFTWARE_BUF_ENABLED)

        tmpStatus = SSD1331_master_GET_STATUS_RX(SSD1331_master_swStatusRx);
        SSD1331_master_swStatusRx = tmpStatus;

        /* Check if RX data FIFO has some data to be moved into the RX Buffer */
        while(0u != (SSD1331_master_swStatusRx & SSD1331_master_STS_RX_FIFO_NOT_EMPTY))
        {
            rxData = CY_GET_REG8(SSD1331_master_RXDATA_PTR);

            /* Set next pointer. */
            SSD1331_master_rxBufferWrite++;
            if(SSD1331_master_rxBufferWrite >= SSD1331_master_RX_BUFFER_SIZE)
            {
                SSD1331_master_rxBufferWrite = 0u;
            }

            if(SSD1331_master_rxBufferWrite == SSD1331_master_rxBufferRead)
            {
                SSD1331_master_rxBufferRead++;
                if(SSD1331_master_rxBufferRead >= SSD1331_master_RX_BUFFER_SIZE)
                {
                    SSD1331_master_rxBufferRead = 0u;
                }

                SSD1331_master_rxBufferFull = 1u;
            }

            /* Move data from the FIFO to the Buffer */
            SSD1331_master_rxBuffer[SSD1331_master_rxBufferWrite] = rxData;

            tmpStatus = SSD1331_master_GET_STATUS_RX(SSD1331_master_swStatusRx);
            SSD1331_master_swStatusRx = tmpStatus;
        }

    #endif /* (SSD1331_master_RX_SOFTWARE_BUF_ENABLED) */

    /* User code required at end of ISR (Optional) */
    /* `#START SSD1331_master_RX_ISR_END` */

    /* `#END` */
    
    #ifdef SSD1331_master_RX_ISR_EXIT_CALLBACK
        SSD1331_master_RX_ISR_ExitCallback();
    #endif /* SSD1331_master_RX_ISR_EXIT_CALLBACK */
}

/* [] END OF FILE */
